﻿import os
import app.servicecontrol
import app.servicemanifest
import app.batch
import app.defaults
import app.logutil
import app.batch
import app.selinux
import re

class FoldersActionMixin(object):
    def applyServicePolicy(self, service):
        if not app.selinux.File.isSELinuxEnabled():
            self.logger.info("Skipping installation of SELinux policies as SELinux is disabled")
            return

        policyFolder = os.path.join(
            app.defaults.DEFAULT_SERVICE_DIR,
            service.getName(),
            "hooks",
            "policy")

        if not os.path.isdir(policyFolder):
            return
        for fileName in os.listdir(policyFolder):
            # fileName extension should be .pp
            if not re.search('\.pp$',fileName):
                continue
            policyFile = os.path.join(policyFolder, fileName)
            if os.path.isfile(policyFile):
                self.logger.info("Loading policy %s of the service %s..." % (policyFile, service.getName()))
                app.selinux.Policy.load(policyFile)

    def applyServiceFolders(self, service, forceFolderRecreation, forceReapplyPermissions):
        #Policy would be getting loaded while running bootstrap and applyPermission not
        #in migration because it may take long time to apply context using semanage command
        #(if they specified in manifest.json) for the folders so loading policy in migration is unnecessary.
        if forceFolderRecreation or forceReapplyPermissions:
            self.applyServicePolicy(service)

        # Run hooks, if any.
        scriptsFolder = os.path.join(
            app.defaults.DEFAULT_SERVICE_DIR,
            service.getName(),
            "hooks",
            "folder")

        if os.path.isdir(scriptsFolder):
            if forceFolderRecreation:
                # Delete...
                app.batch.FolderDeletionBatch(
                    self.logger,
                    app.util.getRuntimeConfiguration()) \
                    .addFolder(scriptsFolder) \
                    .execute()

            # Create...
            app.batch.FolderCreationBatch(
                self.logger,
                app.util.getRuntimeConfiguration()) \
                .addFolder(scriptsFolder) \
                .execute()

            if forceReapplyPermissions:
                # Apply permissions...
                app.batch.FolderPermissionsBatch(
                    self.logger,
                    app.util.getRuntimeConfiguration()) \
                    .addFolder(scriptsFolder) \
                    .execute()

        # Handle folders described in manifest.
        for folder in service.getFolders():
            self.logger.info("Applying %s..." % folder)
            folder.apply(forceFolderRecreation, forceReapplyPermissions)

class ApplyPermissionsAction(app.servicecontrol.ControllerAction, FoldersActionMixin):
    def __init__(self, services, logger):
        self.__services = services
        return super(ApplyPermissionsAction, self).__init__(logger)

    def chain(self, action):
        assert isinstance(action, ApplyPermissionsAction)

        return ApplyPermissionsAction(
            self.__services + action.getInvolvedServices(),
            self.logger)

    def getInvolvedServices(self):
        return self.__services

    def execute(self):
        allOk = True

        for service in self.__services:
            try:
                self.printLog('title', "Applying permissions for %s..." % service.getName())
                self.applyServiceFolders(service, False, True)

            except Exception, e:
                self.printLog("error", str(e))
                self.logger.error(''.join(app.util.getStackTraceAndException()).strip())
                allOk = False

        if not allOk:
            self.printLog("error", "\nSetting permissions failed. See %s for details." % (app.logutil.getFileName(self.logger) or "log"))

        return allOk
