﻿import app.util
import app.servicecontrol

class PrintStatusServiceAction(app.servicecontrol.ControllerAction):
    def __init__(self, services, logger):
        self.__services = services
        self.__enableHeader = True
        return super(PrintStatusServiceAction, self).__init__(logger)

    def setHeaderEnabled(self, enabled):
        self.__enableHeader = enabled

    def getInvolvedServices(self):
        return self.__machine.getServices()

    def execute(self):
        if self.__enableHeader:
            self.printLog("info", "%6s %-40s" % ("PID", "Service"))

        allOk = True

        metastatusMapping = {
            app.servicecontrol.ServiceMetaStatus.OK : "success",
            app.servicecontrol.ServiceMetaStatus.INCONCLUSIVE : "warn",
            app.servicecontrol.ServiceMetaStatus.BAD: "error"
        }

        for service in self.__services:
            controller = service.getController(self.logger)
            status = controller.getStatus()

            allOk = allOk and status.getMetastatus() == app.servicecontrol.ServiceMetaStatus.OK

            self.printLog("info", "%6s %-53s" % (controller.getPid() or "", service.getName()), False)
            self.printLog(metastatusMapping[status.getMetastatus()], "[%s]" % status)

        return allOk
