﻿import os

import app.util
import app.servicecontrol
import app.batch
import app.defaults
import app.logutil
import app.actions.folders

class MigrateServiceAction(app.servicecontrol.ControllerAction, app.actions.folders.FoldersActionMixin):
    def __init__(self, services, logger):
        self.__services = services
        return super(MigrateServiceAction, self).__init__(logger)

    def chain(self, action):
        assert isinstance(action, MigrateServiceAction)

        return MigrateServiceAction(
            self.__services + action.getInvolvedServices(),
            self.logger)

    def getInvolvedServices(self):
        return self.__services

    def confirm(self, isRedeployment=False):
        if not len(self.__services):
            # Empty action - there is nothing to confirm.
            return True

        self.printLog(
            "warn",
            "The following services will be migrated: \n  %s" % (
                    "\n  ".join(sorted([service.getName() for service in self.__services]))))

        return "y" == app.util.readFromUser("Do you wish to continue? [y/N]:", "y", ["y","N"])

    def __migrateService(self, service):
        # (1) Create folder structure (without deleting data or reapplying permissions).
        self.applyServiceFolders(service, False, False)

        # (2) Run migrate hooks.
        #
        # A separate batch is run for each service so that we the order
        # corresponds to the order of the services list

        scriptsFolder = os.path.join(
            app.defaults.DEFAULT_SERVICE_DIR,
            service.getName(),
            "hooks",
            "migrate")

        if os.path.isdir(scriptsFolder):
            self.printLog('title', "Migrating %s..." % service.getName())
            app.batch.MigrateBatch(self.logger, app.util.getRuntimeConfiguration()) \
                .addFolder(scriptsFolder) \
                .execute()

    def execute(self):
        allOk = True

        for service in self.__services:
            try:
                self.__migrateService(service)

            except Exception, e:
                self.printLog("error", str(e))
                self.logger.error(''.join(app.util.getStackTraceAndException()).strip())
                allOk = False

        if not allOk:
            self.printLog("error", "\nMigration failed. See %s for details." % (app.logutil.getFileName(self.logger) or "log"))

        return allOk
