﻿import os

import app.util
import app.defaults
import app.logutil
import app.actions.lockedaction

class ReinitializeServiceAction(app.actions.lockedaction.LockedControllerAction):
    def __init__(self, services, logger):
        self.__services = services
        return super(ReinitializeServiceAction, self).__init__(logger)

    def chain(self, action):
        assert isinstance(action, ReinitializeServiceAction)

        return ReinitializeServiceAction(
            self.__services + action.getInvolvedServices(),
            self.logger)

    def getInvolvedServices(self):
        return self.__services

    def confirm(self):
        return True

    def __reinitializeServices(self):
        allOk = True

        for service in self.__services:
            scriptsFolder = os.path.join(
                app.defaults.DEFAULT_SERVICE_DIR,
                service.getName(),
                "hooks",
                "initialize")

            if os.path.isdir(scriptsFolder):
                self.printLog('title', "Reinitializing %s..." % service.getName())

                batch = app.batch.InitializationBatch(
                    self.logger,
                    app.util.getRuntimeConfiguration())

                batch.addFolder(scriptsFolder)

                try:
                    batch.execute()
                except Exception:
                    self.logger.error(''.join(app.util.getStackTraceAndException()).strip())
                    allOk = False

        if not allOk:
            self.printLog("error", "\nReinitialization failed. See %s for details." % (app.logutil.getFileName(self.logger) or "log"))

        return allOk

    def lockedExecute(self):
        reinitResult = self.__reinitializeServices()
        return  reinitResult
