﻿import os
import app.util
import app.servicecontrol
import app.defaults

class ConfigurationIntegrity:

    def __init__(self, services, checksumConfigFile):
       self.services  = services
       self.confObj = app.util.configParser()
       self.checksumConfigFile = checksumConfigFile

    def checkRuntimeIntegrity(self):
        integrityFailedList = []

        if os.path.isfile(self.checksumConfigFile):
            self.confObj.read(self.checksumConfigFile)

        for service in self.services:
            runtimeConfFiles = service.getIntegrityConfFiles()
            if runtimeConfFiles != None:
                serviceName = service.getName()
                # old checksum config file has no section legacy-runtime
                # change the section to all while parsing the old checksum config file
                if os.path.basename(self.checksumConfigFile) == os.path.basename(app.defaults.OLD_CHECKSUM_CONFIG_FILE) and serviceName == "legacy-runtime":
                    serviceName = "all"

                for fileName in runtimeConfFiles:
                   try:
                       curCheckSum = self.confObj.get(serviceName, fileName)
                       newCheckSum = app.util.getCheckSum(runtimeConfFiles[fileName])
                   except:
                       continue

                   if newCheckSum == None:
                       continue
                   if curCheckSum != newCheckSum:
                       integrityFailedList.append(runtimeConfFiles[fileName])

        return integrityFailedList


    def writeIntegrityChecksums(self):
        configFolder = os.path.dirname(self.checksumConfigFile)
        if not os.path.isdir(configFolder):
            os.makedirs(configFolder)

        for service in self.services:
            runtimeConfigFiles = service.getIntegrityConfFiles()
            if runtimeConfigFiles != None:
                serviceName = service.getName()
                self.confObj.add_section(serviceName)
                for fileName in runtimeConfigFiles:
                    checkSum = app.util.getCheckSum(runtimeConfigFiles[fileName])
                    self.confObj.set(serviceName, fileName, checkSum)

        with open(self.checksumConfigFile, "wb") as cfgFile:
            self.confObj.write(cfgFile)


