import signal
import socket
from contextlib import closing

def isPortListening(host, port, logger, socketTimeout=20):
    def alarmHandler(signum,frame):
        raise ConnectionTimeoutException("Connection timeout")

    logger.info("Checking endpoint %s:%d..." % (host, port))
    try:
        # Install signal handler to avoid connect() from blocking indefinitely
        signal.signal(signal.SIGALRM, alarmHandler)
        signal.alarm(socketTimeout)

        with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
            errno = sock.connect_ex((host, port))
            if errno == 0:
                logger.info("Endpoint %s:%d is available" % (host, port))
                return True
            else:
                logger.error("Endpoint %s:%d not available: %d" % (host, port, errno))
                return False

    except ConnectionTimeoutException:
        logger.error("Connectingt to endpoint %s:%d timed out" % (host, port))
        return False
    except Exception, sockErrMsg:
        if socket.herror:
            logger.error("Host not found %s" % host)
        else:
            logger.error("Connecting to endpoint %s:%d is failed and error is %s" %
                    (host, port, sockErrMsg))
        return False
    finally:
        signal.alarm(0)

class ConnectionTimeoutException(Exception):
    pass
