﻿__copyright__ = "Copyright (c) 1999-2013 CollabNet, Inc. All rights reserved."

import os
import StringIO
import commands

import app.util
import app.landscape

class PGDBAdmin(object):
    def __init__(self, dbname, user, password, readOnlyUser,
                 readOnlyPassword, logger, port):
        self.dbname = dbname
        self.user = user
        self.password = password
        self.readOnlyUser = readOnlyUser
        self.readOnlyPassword = readOnlyPassword
        self.logger = logger
        self.port = port

    def __runAsPostgres(self,cmd):
        pgCmd = "su - postgres -c \"\"\" %s \"\"\"" % cmd
        return app.util.runCommand(pgCmd, logger=self.logger)

    def dropDBIfExists(self):
        dropDbCmd = "dropdb -p %s --if-exists %s" % (self.port, self.dbname )
        self.__runAsPostgres(dropDbCmd)

    def _alterDBPasswd(self, user, password):

        setPasswordCmd = "psql -p %s -c \\\"ALTER USER \\\\\\\"%s\\\\\\\" WITH PASSWORD '%s' \\\"" % (self.port, user, password)
        self.__runAsPostgres(setPasswordCmd)


    def __reCreateUserWithPermission(self, user, password, permission_flags):
        dropOwnedCmd = "psql -p %s -c \\\"DROP OWNED BY \\\\\\\"%s\\\\\\\" \\\"" % (self.port, user)
        try:
            self.__runAsPostgres(dropOwnedCmd)
        except:
            pass

        dropUsrCmd = "dropuser -p %s --if-exists %s" % (self.port, user)
        self.__runAsPostgres(dropUsrCmd)

        createUsrCmd = "createuser -p %s %s %s" %(self.port, ' '.join(permission_flags), user)
        self.__runAsPostgres(createUsrCmd)

        self._alterDBPasswd(user, password)

    def reCreateRWUser(self, user, password):
        return self.__reCreateUserWithPermission(user, password,
            ["--login", "--createdb", "--no-createrole"])

    def reCreateROUser(self, user, password):
        return self.__reCreateUserWithPermission(user, password,
            ["--login", "--no-createdb", "--no-createrole"])

    def reCreateUsers(self):

        try:
            userName = self.user
            self.reCreateRWUser(self.user,self.password)

            if self.readOnlyUser:
                userName = self.readOnlyUser
                self.reCreateROUser(self.readOnlyUser, self.readOnlyPassword)
        except Exception:
            raise Exception(
                    "Dropping the Postgres user %s failed.Please make sure"
                    " that there are no other databases referring to this user" %
                    userName )

    def createDB(self):
        createDBCmd = "createdb -p %s --encoding UTF8  -O %s %s" % (self.port,  self.user, self.dbname)
        self.__runAsPostgres(createDBCmd)

    def isDatabaseExisting(self):

        command = "su - postgres -c \"psql -p %s -tAc 'SELECT datname FROM pg_database' | grep -qw %s\"" % (self.port, self.dbname)
        (exitCode, output) = commands.getstatusoutput(command)

        return exitCode == 0

    def createDBUser(self, permissionFlags):

        createUsrCmd = "createuser -p %s %s %s" % (self.port, ' '.join(permissionFlags), self.user)
        self.__runAsPostgres(createUsrCmd)

    def dropDBUserIfExists(self):

        dropUsrCmd = "dropuser -p %s --if-exists %s" % (self.port, self.user)
        self.__runAsPostgres(dropUsrCmd)

    def setDBPasswd(self):

        self._alterDBPasswd(self.user, self.password)



class PGCTFDBManager(PGDBAdmin):

    def __init__(self, dbname, user, password, readOnlyUser,
                 readOnlyPassword, logger, port,host,passFile):
        super(PGCTFDBManager, self).__init__(dbname, user, password, readOnlyUser, readOnlyPassword, logger, port)
        self.host = host
        self.passFile = passFile

    def __runSQLAsDBAdmin(self, sql):
        pgCmd = """ psql -U'%s' -h'%s' -p'%s' '%s' -c ' %s '  """ % (self.user, self.host,self.port,self.dbname,sql)
        os.environ['PGPASSWORD'] = self.password
        os.environ['PGPASSFILE'] = self.passFile

        return app.util.runCommand(pgCmd, logger=self.logger)

    def grantReadOnlyPermissionsForAllTablesInSchema(self):
        grantPermission = "GRANT SELECT ON ALL TABLES IN SCHEMA public TO \"%s\" " % self.readOnlyUser
        self.__runSQLAsDBAdmin(grantPermission)

        alterDefaultPrivileges = "ALTER DEFAULT PRIVILEGES FOR ROLE \"%s\"  IN SCHEMA public GRANT SELECT ON TABLES TO \"%s\" ;" % (self.user, self.readOnlyUser)
        self.__runSQLAsDBAdmin(alterDefaultPrivileges)

    def revokePermissionForTables(self, tableList):
        revokePermission = "REVOKE ALL ON table %s FROM \"%s\" " % (",".join(tableList ), self.readOnlyUser)
        self.__runSQLAsDBAdmin(revokePermission)

class HbaConfig(object):
    """ Helper class for managing pg_hba.conf """

    def __init__(self):
        self.__buffer = StringIO.StringIO()

    def __repr__(self):
        return self.__buffer.getvalue()

    def format(self):
        return self.__buffer.getvalue()

    def allowAccessForIp(self, databaseName, user, ip, isSsl):
        if not ip:
            raise AttributeError("ip")

        self.__buffer.write("\n%s   %s  %s  %s         255.255.255.255   md5" % (
            "hostssl" if isSsl else "host",
            databaseName,
            user,
            ip))
        return self

    def allowAccessForMachine(self, databaseName, user, machine, isSsl):
        for ip in ["127.0.0.1", machine.getIp()]:
            self.allowAccessForIp(databaseName, user, ip, isSsl)

        return self

    def allowAccessForService(self, databaseName, user, serviceName, isSsl):
        self.__buffer.write("\n# <%s>" % serviceName)
        ips = [machine.getIp() for machine in app.landscape.Landscape.getSite().getMachinesByService(serviceName)]
        ips.append("127.0.0.1")

        for ip in ips:
            self.allowAccessForIp(databaseName, user, ip, isSsl)

        self.__buffer.write("\n# </%s>\n" % serviceName)
        return self
