import os
import app.util
import app.colors

class File(object):
    @staticmethod
    def __run(command):
        exitCode = os.system(command)
        if exitCode != 0:
            raise SelinuxException("Command '%s' failed with exit code %d" % (command, exitCode))

    @staticmethod
    def __addTypeAndRestore(path, type, recursive):
        # It is always better to have absolute paths in
        # /etc/selinux/targeted/contexts/files/file_contexts.local
        path = os.path.abspath(path)

        # Clear recurive path if this is not a directory
        recursive = recursive and os.path.isdir(path)

        # Add file context.
        if type:
            # semanage command will work only when selinux mode either permissive or Enforce mode
            if File.isSELinuxEnabled():
                File.__run("semanage fcontext -a -t %s '%s%s'" % (
                    type,
                    path,
                    "(/.*)?" if recursive else ""))
            else:
                print app.colors.Color.yellow("  SElinux is disabled, so SELinux context will not be applied for path %s" % path)

        # Run restorecon to have above change take effect.
        File.__run("/sbin/restorecon %s %s" % (
            "-R" if recursive else "",
            path))

    @staticmethod
    def getTypeContext(path):
        fileContext = app.util.executeCommand("matchpathcon -n %s" % (path))
        return fileContext.split(":")[2]

    @staticmethod
    def isSELinuxEnabled():
        selinuxMode = app.util.executeCommand("getenforce")
        if selinuxMode.strip() == "Disabled":
            return False
        return True

    @staticmethod
    def setContext(path, type, recursive=True):
        if os.path.islink(path):
            # The symlink must be treated separately from the target
            File.__addTypeAndRestore(path, type, False)
            originalPath = os.readlink(path)
            if os.path.islink(originalPath):
                File.__addTypeAndRestore(originalPath, type, False)
            if type == None:
                type = File.getTypeContext(path)
            File.__addTypeAndRestore(os.path.realpath(path), type, recursive)
        else:
            File.__addTypeAndRestore(path, type, recursive)

class Policy(object):
    @staticmethod
    def __run(command):
        exitCode = os.system(command)
        if exitCode != 0:
            raise SelinuxException("Command '%s' failed with exit code %d" % (command, exitCode))
    @staticmethod
    def load(policyFile):
        cmd = "semodule -i %s" % (policyFile)
        Policy.__run(cmd)

class SelinuxException(Exception):
    pass
