__copyright__ = "Copyright (c) 2014 CollabNet, Inc. All rights reserved."

from app.scripts import BaseInitializationScript
import app.util
import app.landscape
from app.colors import Color
import os
import commands
import shutil
import socket

class InitializationScript(BaseInitializationScript):

    scriptDescription = "Import SSL certificates of integration servers"

    def __trustHost(self, host, port):
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect((host, port))
        except socket.error:
            self.logMesg("warn", Color.yellow(
           "\n  Importing the SSL certificates of SCM servers failed as the respective services may not be fully up yet."+
           "\n  You can run /opt/collabnet/teamforge/bin/teamforge reinitialize to retry the operation."))
            return

        return app.util.executeCommand(" ".join([
            os.path.join(self.cfg.get('RUNTIME_SCRIPTS'), "trust-cert.sh"),
            host,
            str(port)
            ])) == 0

    def __trustScmServers(self):
        if (self.cfg.get('SSL', 'off') != 'on'):
            self.logMesg("debug", "SSL not enabled, importing SSL certificates is not necessary");
            return

        for server in app.landscape.Landscape.getSite().getMachinesByService("integration"):
            endpoint = server.getService("integration").getProvidedEndpoint("http-soap", "public")

            self.log.info("Importing SSL certificate from %s:%d" % (endpoint.getHost(), endpoint.getPort()))
            self.__trustHost(endpoint.getHost(), endpoint.getPort())

        for server in app.landscape.Landscape.getSite().getMachinesByService("gerrit"):
            endpoint = server.getService("gerrit").getProvidedEndpoint("http-web", "public")

            if not server.isLocal(): #there is not point in importing localhost certs
                self.log.info("Importing SSL certificate Gerrit from %s:%d" % (endpoint.getHost(), endpoint.getPort()))
                self.__trustHost(endpoint.getHost(), endpoint.getPort())


    def initialize(self):
        self.__trustScmServers()
