﻿__copyright__ = "Copyright (c) 2014 CollabNet, Inc. All rights reserved."

import os
import sys

from app.scripts import BaseInitializationScript
import app.util
import app.db

PITCHFORK = os.path.join(app.defaults.DEFAULT_SERVICE_DIR, "project-templates", "bin", "pitchfork.py")
TEMPLATE_FOLDER = os.path.join(app.defaults.DEFAULT_SERVICE_DIR, "project-templates", "templates")

class InitializationScript(BaseInitializationScript):

    scriptDescription = "Install project templates"

    def initialize(self):
        if self.cfg.get("INSTALL_TEMPLATES", "false") == "true":
            for tpl in ProjectTemplate.loadAllFromFolder(TEMPLATE_FOLDER):
               self.log.info("Installing template %s..." % tpl)
               output = tpl.install("scmadmin", self.cfg["SCM_ADMIN_PASSWORD"])
               self.log.info("Template %s installed: %s" % (tpl, output))


class ProjectTemplate(object):
    def __init__(self, templateArchive):
        self.__projectPath = os.path.basename(templateArchive)[:-4]
        self.__templateArchive = templateArchive

    def __repr__(self):
        return self.__projectPath

    @staticmethod
    def loadAllFromFolder(folderPath):
        return [ProjectTemplate(os.path.join(folderPath, tgzFileName))
                for tgzFileName in os.listdir(folderPath) if tgzFileName.endswith(".tgz")]

    def isInstalled(self):
        connection = None
        try:
            connection = app.db.getConnection()
            projectsWithPath = app.db.fetchAll(
                connection,
                "select id from project where path like '%s%%'" % self.__projectPath)

            return len(projectsWithPath) > 0

        finally:
            if connection:
                app.db.releaseConnection(connection)

    def install(self, user, password, force=False):
        if not force and self.isInstalled():
            # Installed already, nothing to do.
            return "(installed already)"

        command = [
            PITCHFORK,
            "-u", user,
            "-p", password,
            "-c", self.__templateArchive
            ]

        return app.util.executeCommand(command, useShell=False)

if __name__ == "__main__":
    for tpl in ProjectTemplate.loadAllFromFolder(TEMPLATE_FOLDER):
        print "%s, installed: %s" % (tpl, tpl.isInstalled())
        tpl.install(sys.argv[1], sys.argv[2])
