__copyright__ = "Copyright (c) 2016 CollabNet, Inc. All rights reserved."

from app.scripts import BaseCreateRuntimeScript
import app.util
import app.batch
import app.preprocessor
import app.defaults
import commands
import os

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Create collabnet init script"

    def createRuntime(self):

        site_dir = self.cfg["SITE_DIR"]
        startup_file = os.path.join(site_dir,
                  'runtime', 'scripts', 'collabnet')
        linkFile = "/etc/init.d/collabnet"

        # Do os specific stuff.
        # Getting the status of collabnet service during bootup process.

        # Create symlink file for rhel6 alone  before if not
        # create-runtime will fail in fresh install.
        if "redhat-6" in self.cfg['PLATFORM']:
           if os.path.exists(linkFile) or os.path.islink(linkFile):
              cmdOutput = os.remove(linkFile)

           # create a symlink to /etc/init.d
           cmdOutput = os.symlink(startup_file, linkFile)

        if "redhat-7" in self.cfg['PLATFORM']:
           initChkCmd = 'systemctl is-enabled collabnet'
        else:
           initChkCmd = ("/sbin/chkconfig --list collabnet | awk '{print $5}'"
                                                           "| cut -d ':' -f 2")
        initChkStatus = commands.getoutput(initChkCmd)

        # Enable the service only during fresh install

        if not (initChkStatus in ["disabled", "enabled", "on", "off"] and
                                                      os.path.islink(linkFile)):
            if "redhat-7" in self.cfg['PLATFORM']:
                initEnableCmd = 'systemctl enable collabnet &> /dev/null'
            else:
                initEnableCmd = '/sbin/chkconfig --add collabnet &> /dev/null'
            # add appropriate runlevel links
            try:
                cmdOutput = app.util.runSystem(initEnableCmd, self.log)
            except:
                raise Exception("Problem encountered in setting up init script")

        # make sure the scripts exist under <install_dir>/runtime/scripts
        # and remove old ones, if any
        if os.path.exists(linkFile) or os.path.islink(linkFile):
            cmdOutput = os.remove(linkFile)

        # create a symlink to /etc/init.d
        cmdOutput = os.symlink(startup_file, linkFile)
