__copyright__ = "Copyright (c) 2014 CollabNet, Inc. All rights reserved."

from app.scripts import BasePostInstallScript
import app.util
from app.selinux_permissions import SELinuxSetting
import os
import commands
import time
import subprocess
import select

class PostInstallScript(BasePostInstallScript):

    scriptDescription = "Teamforge CLI Server Setup"

    cliServerRPM  = "CN-cliserver";

    def canRun(self):
        if (self.checkPkgInstalled(self.cliServerRPM) != 0):
           self.logMesg("debug", "Teamforge CLI Server is not installed. So skipping CLI Server post installation.")
           self.setSkipMsg("Package %s is not installed"%self.cliServerRPM)
           return False
        return True

    def postInstall(self):
        self.handleCLIServer()
        if self.cfg['SELINUX_ENABLED'] == "true":
           self.handleCLIServerSELinuxPerms()
        try:
          confFilePath = self.cfg['HTTPD_CONF_FILE_PATH']
          if os.path.isfile(confFilePath):
             confFile     = os.path.basename(confFilePath)

             import service.runtime_configfiles
             service.runtime_configfiles.setTFMD5Config('apache', confFile, confFilePath)
        except:
          pass
        self.restartServices()

    def restartServices(self):
        """ Restart CLI Server and Apache service """
        self.logMesg("debug", "In function restartServices");

        initCliServerScript = "/etc/init.d/teamforge_cli_server"
        if "redhat-7" in self.cfg["PLATFORM"]:
            oldCliServerPKG  = os.path.join(self.cfg['SITE_DIR'] , "etc", "add-ons", "teamforge_cli_server.pkg-removed")
            if os.path.isfile(oldCliServerPKG):
                # teamforge_cli_server.service file has changed with new package
                # Load the latest service files from disk
                self.logMesg("debug", "Running systemctl daemon-reload")
                os.system("/bin/systemctl daemon-reload > /dev/null 2>&1")
            tfSystemd = os.path.join(self.cfg["RUNTIME_SCRIPTS"], 'tfsystemd')
            initCliServerScript = "%s teamforge_cli_server"%tfSystemd
        else:
            if not (os.path.isfile(initCliServerScript)):
                return
        self.logMesg("debug", "Restarting Teamforge CLI Server...")
        os.system("%s restart > /dev/null 2>&1" %initCliServerScript)
        self.logMesg("debug", "Restarting apache...")
        os.system("%s restart > /dev/null 2>&1" %self.cfg['HTTPD_SERVICE_CMD'])

        return

    def handleCLIServer(self):
        """ Handles CLI Server Post Installation
              Check if CLI package is installed. If not skip
              Check if CLI package is configured by checking /opt/collabnet/teamforge/etc/addons/CN-cliserver.pkg
                  If found then just run install --runtime --force
                  If not found then run install --install --force
        """

        self.logMesg("debug", "In function handleCLIServer");
        cliServerPKG  = os.path.join(self.cfg['SITE_DIR'] , "etc", "add-ons", "CN-cliserver.pkg")
        cliServerPath = os.path.join(self.cfg['SITE_DIR'], "add-ons", "CN-cliserver")
        cliServerVarPath = os.path.join(self.cfg['DATA_DIR'], "cliserver")

        if (not os.path.isdir(cliServerVarPath) or not os.path.isfile(cliServerPKG)):
           self.cliServerIntegrate(cliServerPath)
        self.cliServerRuntime(cliServerPath)

        return

    def handleCLIServerSELinuxPerms(self):
        """ Handles SELinux permissions for CLI Server directories """
        self.logMesg("debug", "Setting SELinux permissions for CLI Server");
        cliServerPath = os.path.join(self.cfg['SITE_DIR'], "add-ons", "CN-cliserver")
        updateScript = os.path.join(cliServerPath, 'scripts', 'update.pl')
        cliVarPath = os.path.join(self.cfg['DATA_DIR'], "cliserver", "cli")
        selinuxPathList  = [updateScript, cliVarPath]
        selinuxsetting = SELinuxSetting()
        for path in selinuxPathList:
            selinuxsetting.changeContext('httpd_sys_content_t', path)

    def cliServerIntegrate(self, cliServerPath):

        cliServerLog  = os.path.join(self.cfg['SITE_DIR'], "log", "add-ons", "CN-cliserver.log")

        self.logMesg("debug", "Integrating Teamforge CLI Server with Teamforge ...")
        self.logMesg("debug", "----------------------------------------------------------")

        # Invoke CLI Server install command
        installCmd = "cd %s; ./install --force"%cliServerPath
        cliProc = subprocess.Popen(installCmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        time.sleep(1)

        # Wait for CLI Server integration completion
        while True:
           dispData = self.readProcData(cliProc.stdout)
           cliProc.stdout.flush()
           self.logMesg("debug", dispData)
           if cliProc.poll() != None:
              break;

        # Verify CLI Server integration status
        if cliProc.returncode == 0:
           self.logMesg("debug", "Integrating Teamforge CLI Server with Teamforge is successful.")
        else:
           self.addError("Error: While integrating Teamforge CLI Server with Teamforge.\n")
           self.addError("Please refer to /opt/collabnet/teamforge/log/add-ons/CN-cliserver.log for more information\n")
           raise Exception("CLIServerIntegrationException")

        return

    def readProcData(self, fObj):

        (r, w, x) = select.select([fObj], [], [], 0.5)
        value = ""
        while r:
            char = fObj.read(1)
            if (len(char) == 0):
               break
            value += char;
            (r, w, x) = select.select([fObj], [], [], 0.5)

        return value;

    def cliServerRuntime(self, cliServerPath):

        self.logMesg("debug", "Running Teamforge CLI Server runtime creation ...")
        installCmd = "cd %s; ./install --runtime --force"%cliServerPath
        (retVal, outputVal) = commands.getstatusoutput(installCmd)
        if (retVal != 0):
           self.addError("Error: Teamforge CLI Server runtime creation failed.\n")
           self.addError("Reason: [%s]\n"%outputVal)
           raise Exception("CLIServerRuntimeException")
        else:
           self.logMesg("debug", "----------------------------------------------------------")
           self.logMesg("debug", "%s\n"%outputVal)
           self.logMesg("debug", "----------------------------------------------------------\n")
           self.logMesg("debug", "Teamforge CLI Server runtime creation successful.\n")

        return


    def checkPkgInstalled(self, packageName):
        rpmCheckCmd  = "rpm -qi %s"%(packageName)
        (retVal, outputVal) = commands.getstatusoutput(rpmCheckCmd)
        return retVal


