#!/usr/bin/env python

"""This script's sole purpose is to determine whether or not a Service is running, and return a true (0) or false (1) result code. This is determined by the following checks
   a) If Service is available
   b) If Service is running
   If the two conditions are true, then the service is running
"""
import sys
import app.util
import app.landscape
import app.defaults
import app.servicecontrol
import app.logutil

def main(argv):
    if len(argv) < 3:
        sys.stderr.write("Usage: ./is_service_running.py <servicename> <hostname>\n")
        sys.exit(2)

    serviceName = argv[1]
    hostName = argv[2]

    # Load landscape information from runtime-options.conf
    app.landscape.Landscape.initializeFromSiteOptionsPath(app.defaults.RUNTIME_OPTIONS_CONF)

    machine = app.landscape.Landscape.getSite().getMachine(hostName)
    if not machine:
        print "Unknown machine '%s'" % hostName
        sys.exit(2)

    service = machine.getService(serviceName)
    if not service:
        print "Service '%s' is not deployed on machine '%s'" % (serviceName, hostName)
        sys.exit(2)

    if service.getController(app.logutil.getNullLogger()).getStatus().getMetastatus() == app.servicecontrol.ServiceMetaStatus.OK:
        sys.exit(0)
    else:
        sys.exit(1)

if __name__ == '__main__':
        main(sys.argv)

