
from app.scripts import BaseCreateRuntimeScript
import app.util
import app.batch
import service.util
import service.constants
import os

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Create symbolic link for pgsql data dir"

    def createRuntime(self):

        self.createPgsqlSymlink()

    def createPgsqlSymlink(self):

        cfgPgsqlDataDir = self.cfg["PGSQL_DATA_DIR"]

        if os.path.exists(cfgPgsqlDataDir):
            return

        oldDataDirList = ( "/var/lib/pgsql/9.3/data" , "/var/lib/pgsql/9.2/data" )
        cfgPgHomeDir = self.cfg['PGSQL_HOME_DIR']

        self.checkAndCreateSymlink(cfgPgHomeDir, cfgPgsqlDataDir, oldDataDirList)

    def checkAndCreateSymlink(self, homeDir, dataDir, oldDataDirList):

        if not os.path.isdir(homeDir):
            app.util.runSystem("mkdir -p %s" % homeDir, self.log)
            app.util.runSystem("chown -R postgres:postgres %s" % homeDir ,
                    self.log)

        oldDataDir = self.getOldDataDir(oldDataDirList)
        if not oldDataDir:
            os.mkdir(dataDir)
            app.util.runSystem("chown -R postgres:postgres %s" % dataDir ,
                self.log)
            return

        self.createSoftLink(oldDataDir, dataDir)

    def getOldDataDir(self, oldDataDirList):

        oldDataDir = None

        for dataDir in oldDataDirList:
            if os.path.isdir(dataDir) and os.listdir(dataDir):
                oldDataDir = dataDir
                break

        return oldDataDir

    def createSoftLink(self, originalPath, symlinkPath):

        symlinkPath = symlinkPath.rstrip("/")
        cmd = "chown -h postgres:postgres %s"  % symlinkPath

        os.symlink(originalPath, symlinkPath)
        app.util.runSystem(cmd,self.log)

